@@ -1,11 +1,10 @@
 /*!
- * Copyright (c) 2017-2020 by Contributors
+ * Copyright (c) 2017-2021 by Contributors
  * \file c_api.cc
  * \author Hyunsu Cho
  * \brief C API of treelite, used for interfacing with other languages
  */
 
-
 #include <treelite/annotator.h>
 #include <treelite/c_api.h>
 #include <treelite/c_api_error.h>
@@ -17,32 +16,21 @@
 #include <treelite/tree.h>
 #include <treelite/math.h>
 #include <treelite/gtil.h>
-#include <dmlc/thread_local.h>
+#include <treelite/logging.h>
 #include <memory>
 #include <algorithm>
+#include <fstream>
+#include <cstdio>
 
 using namespace treelite;
 
-namespace {
-
-struct CompilerHandleImpl {
-  std::string name;
-  std::vector<std::pair<std::string, std::string>> cfg;
-  std::unique_ptr<Compiler> compiler;
-  explicit CompilerHandleImpl(const std::string& name)
-    : name(name), cfg(), compiler(nullptr) {}
-  ~CompilerHandleImpl() = default;
-};
-
-}  // anonymous namespace
-
 int TreeliteAnnotateBranch(
     ModelHandle model, DMatrixHandle dmat, int nthread, int verbose, AnnotationHandle* out) {
   API_BEGIN();
   std::unique_ptr<BranchAnnotator> annotator{new BranchAnnotator()};
   const Model* model_ = static_cast<Model*>(model);
   const auto* dmat_ = static_cast<const DMatrix*>(dmat);
-  CHECK(dmat_) << "Found a dangling reference to DMatrix";
+  TREELITE_CHECK(dmat_) << "Found a dangling reference to DMatrix";
   annotator->Annotate(*model_, dmat_, nthread, verbose);
   *out = static_cast<AnnotationHandle>(annotator.release());
   API_END();
@@ -52,8 +40,8 @@ int TreeliteAnnotationSave(AnnotationHandle handle,
                            const char* path) {
   API_BEGIN();
   const BranchAnnotator* annotator = static_cast<BranchAnnotator*>(handle);
-  std::unique_ptr<dmlc::Stream> fo(dmlc::Stream::Create(path, "w"));
-  annotator->Save(fo.get());
+  std::ofstream fo(path);
+  annotator->Save(fo);
   API_END();
 }
 
@@ -63,67 +51,36 @@ int TreeliteAnnotationFree(AnnotationHandle handle) {
   API_END();
 }
 
-int TreeliteCompilerCreate(const char* name,
-                           CompilerHandle* out) {
+int TreeliteCompilerCreateV2(const char* name, const char* params_json_str, CompilerHandle* out) {
   API_BEGIN();
-  std::unique_ptr<CompilerHandleImpl> compiler{new CompilerHandleImpl(name)};
+  std::unique_ptr<Compiler> compiler{Compiler::Create(name, params_json_str)};
   *out = static_cast<CompilerHandle>(compiler.release());
   API_END();
 }
 
-int TreeliteCompilerSetParam(CompilerHandle handle,
-                             const char* name,
-                             const char* value) {
+int TreeliteCompilerGenerateCodeV2(CompilerHandle compiler,
+                                   ModelHandle model,
+                                   const char* dirpath) {
   API_BEGIN();
-  CompilerHandleImpl* impl = static_cast<CompilerHandleImpl*>(handle);
-  auto& cfg_ = impl->cfg;
-  std::string name_(name);
-  std::string value_(value);
-  // check for duplicate parameters
-  auto it = std::find_if(cfg_.begin(), cfg_.end(),
-    [&name_](const std::pair<std::string, std::string>& x) {
-      return x.first == name_;
-    });
-  if (it == cfg_.end()) {
-    cfg_.emplace_back(name_, value_);
-  } else {
-    it->second = value;
-  }
-  API_END();
-}
-
-int TreeliteCompilerGenerateCode(CompilerHandle compiler,
-                                 ModelHandle model,
-                                 int verbose,
-                                 const char* dirpath) {
-  API_BEGIN();
-  if (verbose > 0) {  // verbose enabled
-    int ret = TreeliteCompilerSetParam(compiler, "verbose",
-                                       std::to_string(verbose).c_str());
-    if (ret < 0) {  // SetParam failed
-      return ret;
-    }
-  }
   const Model* model_ = static_cast<Model*>(model);
-  CompilerHandleImpl* impl = static_cast<CompilerHandleImpl*>(compiler);
+  Compiler* compiler_ = static_cast<Compiler*>(compiler);
+  TREELITE_CHECK(model_);
+  TREELITE_CHECK(compiler_);
+  compiler::CompilerParam param = compiler_->QueryParam();
 
   // create directory named dirpath
   const std::string& dirpath_(dirpath);
   filesystem::CreateDirectoryIfNotExist(dirpath);
 
-  compiler::CompilerParam cparam;
-  cparam.Init(impl->cfg, dmlc::parameter::kAllMatch);
-
   /* compile model */
-  impl->compiler.reset(Compiler::Create(impl->name, cparam));
-  auto compiled_model = impl->compiler->Compile(*model_);
-  if (verbose > 0) {
-    LOG(INFO) << "Code generation finished. Writing code to files...";
+  auto compiled_model = compiler_->Compile(*model_);
+  if (param.verbose > 0) {
+    TREELITE_LOG(INFO) << "Code generation finished. Writing code to files...";
   }
 
   for (const auto& it : compiled_model.files) {
-    if (verbose > 0) {
-      LOG(INFO) << "Writing file " << it.first << "...";
+    if (param.verbose > 0) {
+      TREELITE_LOG(INFO) << "Writing file " << it.first << "...";
     }
     const std::string filename_full = dirpath_ + "/" + it.first;
     if (it.second.is_binary) {
@@ -138,7 +95,7 @@ int TreeliteCompilerGenerateCode(CompilerHandle compiler,
 
 int TreeliteCompilerFree(CompilerHandle handle) {
   API_BEGIN();
-  delete static_cast<CompilerHandleImpl*>(handle);
+  delete static_cast<Compiler*>(handle);
   API_END();
 }
 
@@ -232,7 +189,7 @@ int TreeliteLoadSKLearnGradientBoostingClassifier(
 int TreeliteSerializeModel(const char* filename, ModelHandle handle) {
   API_BEGIN();
   FILE* fp = std::fopen(filename, "wb");
-  CHECK(fp) << "Failed to open file '" << filename << "'";
+  TREELITE_CHECK(fp) << "Failed to open file '" << filename << "'";
   auto* model_ = static_cast<Model*>(handle);
   model_->SerializeToFile(fp);
   std::fclose(fp);
@@ -242,7 +199,7 @@ int TreeliteSerializeModel(const char* filename, ModelHandle handle) {
 int TreeliteDeserializeModel(const char* filename, ModelHandle* out) {
   API_BEGIN();
   FILE* fp = std::fopen(filename, "rb");
-  CHECK(fp) << "Failed to open file '" << filename << "'";
+  TREELITE_CHECK(fp) << "Failed to open file '" << filename << "'";
   std::unique_ptr<Model> model = Model::DeserializeFromFile(fp);
   std::fclose(fp);
   *out = static_cast<ModelHandle>(model.release());
@@ -294,10 +251,10 @@ int TreeliteQueryNumClass(ModelHandle handle, size_t* out) {
 
 int TreeliteSetTreeLimit(ModelHandle handle, size_t limit) {
   API_BEGIN();
-  CHECK_GT(limit, 0) << "limit should be greater than 0!";
+  TREELITE_CHECK_GT(limit, 0) << "limit should be greater than 0!";
   auto* model_ = static_cast<Model*>(handle);
   const size_t num_tree = model_->GetNumTree();
-  CHECK_GE(num_tree, limit) << "Model contains less trees(" << num_tree << ") than limit";
+  TREELITE_CHECK_GE(num_tree, limit) << "Model contains fewer trees(" << num_tree << ") than limit";
   model_->SetTreeLimit(limit);
   API_END();
 }
@@ -336,7 +293,7 @@ int TreeliteDeleteTreeBuilder(TreeBuilderHandle handle) {
 int TreeliteTreeBuilderCreateNode(TreeBuilderHandle handle, int node_key) {
   API_BEGIN();
   auto* builder = static_cast<frontend::TreeBuilder*>(handle);
-  CHECK(builder) << "Detected dangling reference to deleted TreeBuilder object";
+  TREELITE_CHECK(builder) << "Detected dangling reference to deleted TreeBuilder object";
   builder->CreateNode(node_key);
   API_END();
 }
@@ -344,7 +301,7 @@ int TreeliteTreeBuilderCreateNode(TreeBuilderHandle handle, int node_key) {
 int TreeliteTreeBuilderDeleteNode(TreeBuilderHandle handle, int node_key) {
   API_BEGIN();
   auto* builder = static_cast<frontend::TreeBuilder*>(handle);
-  CHECK(builder) << "Detected dangling reference to deleted TreeBuilder object";
+  TREELITE_CHECK(builder) << "Detected dangling reference to deleted TreeBuilder object";
   builder->DeleteNode(node_key);
   API_END();
 }
@@ -352,7 +309,7 @@ int TreeliteTreeBuilderDeleteNode(TreeBuilderHandle handle, int node_key) {
 int TreeliteTreeBuilderSetRootNode(TreeBuilderHandle handle, int node_key) {
   API_BEGIN();
   auto* builder = static_cast<frontend::TreeBuilder*>(handle);
-  CHECK(builder) << "Detected dangling reference to deleted TreeBuilder object";
+  TREELITE_CHECK(builder) << "Detected dangling reference to deleted TreeBuilder object";
   builder->SetRootNode(node_key);
   API_END();
 }
@@ -362,7 +319,7 @@ int TreeliteTreeBuilderSetNumericalTestNode(
     ValueHandle threshold, int default_left, int left_child_key, int right_child_key) {
   API_BEGIN();
   auto* builder = static_cast<frontend::TreeBuilder*>(handle);
-  CHECK(builder) << "Detected dangling reference to deleted TreeBuilder object";
+  TREELITE_CHECK(builder) << "Detected dangling reference to deleted TreeBuilder object";
   builder->SetNumericalTestNode(node_key, feature_id, opname,
                                 *static_cast<const frontend::Value*>(threshold),
                                 (default_left != 0), left_child_key, right_child_key);
@@ -375,10 +332,10 @@ int TreeliteTreeBuilderSetCategoricalTestNode(
     int left_child_key, int right_child_key) {
   API_BEGIN();
   auto* builder = static_cast<frontend::TreeBuilder*>(handle);
-  CHECK(builder) << "Detected dangling reference to deleted TreeBuilder object";
+  TREELITE_CHECK(builder) << "Detected dangling reference to deleted TreeBuilder object";
   std::vector<uint32_t> vec(left_categories_len);
   for (size_t i = 0; i < left_categories_len; ++i) {
-    CHECK(left_categories[i] <= std::numeric_limits<uint32_t>::max());
+    TREELITE_CHECK(left_categories[i] <= std::numeric_limits<uint32_t>::max());
     vec[i] = static_cast<uint32_t>(left_categories[i]);
   }
   builder->SetCategoricalTestNode(node_key, feature_id, vec, (default_left != 0),
@@ -389,7 +346,7 @@ int TreeliteTreeBuilderSetCategoricalTestNode(
 int TreeliteTreeBuilderSetLeafNode(TreeBuilderHandle handle, int node_key, ValueHandle leaf_value) {
   API_BEGIN();
   auto* builder = static_cast<frontend::TreeBuilder*>(handle);
-  CHECK(builder) << "Detected dangling reference to deleted TreeBuilder object";
+  TREELITE_CHECK(builder) << "Detected dangling reference to deleted TreeBuilder object";
   builder->SetLeafNode(node_key, *static_cast<const frontend::Value*>(leaf_value));
   API_END();
 }
@@ -398,11 +355,11 @@ int TreeliteTreeBuilderSetLeafVectorNode(TreeBuilderHandle handle, int node_key,
                                          const ValueHandle* leaf_vector, size_t leaf_vector_len) {
   API_BEGIN();
   auto* builder = static_cast<frontend::TreeBuilder*>(handle);
-  CHECK(builder) << "Detected dangling reference to deleted TreeBuilder object";
+  TREELITE_CHECK(builder) << "Detected dangling reference to deleted TreeBuilder object";
   std::vector<frontend::Value> vec(leaf_vector_len);
-  CHECK(leaf_vector) << "leaf_vector argument must not be null";
+  TREELITE_CHECK(leaf_vector) << "leaf_vector argument must not be null";
   for (size_t i = 0; i < leaf_vector_len; ++i) {
-    CHECK(leaf_vector[i]) << "leaf_vector[" << i << "] contains an empty Value handle";
+    TREELITE_CHECK(leaf_vector[i]) << "leaf_vector[" << i << "] contains an empty Value handle";
     vec[i] = *static_cast<const frontend::Value*>(leaf_vector[i]);
   }
   builder->SetLeafVectorNode(node_key, vec);
@@ -424,7 +381,7 @@ int TreeliteModelBuilderSetModelParam(ModelBuilderHandle handle, const char* nam
                                       const char* value) {
   API_BEGIN();
   auto* builder = static_cast<frontend::ModelBuilder*>(handle);
-  CHECK(builder) << "Detected dangling reference to deleted ModelBuilder object";
+  TREELITE_CHECK(builder) << "Detected dangling reference to deleted ModelBuilder object";
   builder->SetModelParam(name, value);
   API_END();
 }
@@ -439,9 +396,9 @@ int TreeliteModelBuilderInsertTree(ModelBuilderHandle handle, TreeBuilderHandle
                                    int index) {
   API_BEGIN();
   auto* model_builder = static_cast<frontend::ModelBuilder*>(handle);
-  CHECK(model_builder) << "Detected dangling reference to deleted ModelBuilder object";
+  TREELITE_CHECK(model_builder) << "Detected dangling reference to deleted ModelBuilder object";
   auto* tree_builder = static_cast<frontend::TreeBuilder*>(tree_builder_handle);
-  CHECK(tree_builder) << "Detected dangling reference to deleted TreeBuilder object";
+  TREELITE_CHECK(tree_builder) << "Detected dangling reference to deleted TreeBuilder object";
   return model_builder->InsertTree(tree_builder, index);
   API_END();
 }
@@ -449,9 +406,9 @@ int TreeliteModelBuilderInsertTree(ModelBuilderHandle handle, TreeBuilderHandle
 int TreeliteModelBuilderGetTree(ModelBuilderHandle handle, int index, TreeBuilderHandle *out) {
   API_BEGIN();
   auto* model_builder = static_cast<frontend::ModelBuilder*>(handle);
-  CHECK(model_builder) << "Detected dangling reference to deleted ModelBuilder object";
+  TREELITE_CHECK(model_builder) << "Detected dangling reference to deleted ModelBuilder object";
   auto* tree_builder = model_builder->GetTree(index);
-  CHECK(tree_builder) << "Detected dangling reference to deleted TreeBuilder object";
+  TREELITE_CHECK(tree_builder) << "Detected dangling reference to deleted TreeBuilder object";
   *out = static_cast<TreeBuilderHandle>(tree_builder);
   API_END();
 }
@@ -459,7 +416,7 @@ int TreeliteModelBuilderGetTree(ModelBuilderHandle handle, int index, TreeBuilde
 int TreeliteModelBuilderDeleteTree(ModelBuilderHandle handle, int index) {
   API_BEGIN();
   auto* builder = static_cast<frontend::ModelBuilder*>(handle);
-  CHECK(builder) << "Detected dangling reference to deleted ModelBuilder object";
+  TREELITE_CHECK(builder) << "Detected dangling reference to deleted ModelBuilder object";
   builder->DeleteTree(index);
   API_END();
 }
@@ -467,7 +424,7 @@ int TreeliteModelBuilderDeleteTree(ModelBuilderHandle handle, int index) {
 int TreeliteModelBuilderCommitModel(ModelBuilderHandle handle, ModelHandle* out) {
   API_BEGIN();
   auto* builder = static_cast<frontend::ModelBuilder*>(handle);
-  CHECK(builder) << "Detected dangling reference to deleted ModelBuilder object";
+  TREELITE_CHECK(builder) << "Detected dangling reference to deleted ModelBuilder object";
   std::unique_ptr<Model> model = builder->CommitModel();
   *out = static_cast<ModelHandle>(model.release());
   API_END();
