@@ -1,27 +1,32 @@
 /*!
- * Copyright (c) 2017-2020 by Contributors
+ * Copyright (c) 2017-2021 by Contributors
  * \file ast_native.cc
- * \author Hyunsu Cho
  * \brief C code generator
+ * \author Hyunsu Cho
  */
 #include <treelite/compiler.h>
 #include <treelite/compiler_param.h>
 #include <treelite/annotator.h>
 #include <fmt/format.h>
+#include <rapidjson/stringbuffer.h>
+#include <rapidjson/writer.h>
 #include <algorithm>
 #include <fstream>
 #include <unordered_map>
 #include <queue>
+#include <cstdio>
 #include <cmath>
+#include <cstdint>
+#include "./ast_native.h"
 #include "./pred_transform.h"
 #include "./ast/builder.h"
 #include "./native/main_template.h"
 #include "./native/header_template.h"
 #include "./native/qnode_template.h"
 #include "./native/code_folder_template.h"
+#include "./native/typeinfo_ctypes.h"
 #include "./common/format_util.h"
 #include "./common/code_folding_util.h"
-#include "./common/categorical_bitmap.h"
 
 #if defined(_MSC_VER) || defined(_WIN32)
 #define DLLEXPORT_KEYWORD "__declspec(dllexport) "
@@ -34,53 +39,46 @@ using namespace fmt::literals;
 namespace treelite {
 namespace compiler {
 
-DMLC_REGISTRY_FILE_TAG(ast_native);
-
-class ASTNativeCompiler : public Compiler {
+class ASTNativeCompilerImpl {
  public:
-  explicit ASTNativeCompiler(const CompilerParam& param)
-    : param(param) {
-    if (param.verbose > 0) {
-      LOG(INFO) << "Using ASTNativeCompiler";
-    }
-    if (param.dump_array_as_elf > 0) {
-      LOG(INFO) << "Warning: 'dump_array_as_elf' parameter is not applicable "
-                   "for ASTNativeCompiler";
-    }
-  }
+  explicit ASTNativeCompilerImpl(const CompilerParam& param) : param_(param) {}
 
-  CompiledModel Compile(const Model& model) override {
+  template <typename ThresholdType, typename LeafOutputType>
+  CompiledModel CompileImpl(const ModelImpl<ThresholdType, LeafOutputType>& model) {
     CompiledModel cm;
     cm.backend = "native";
 
+    TREELITE_CHECK(model.task_type != TaskType::kMultiClfCategLeaf)
+      << "Model task type unsupported by ASTNativeCompiler";
+    TREELITE_CHECK(model.task_param.output_type == TaskParam::OutputType::kFloat)
+      << "ASTNativeCompiler only supports models with float output";
+
     num_feature_ = model.num_feature;
-    num_output_group_ = model.num_output_group;
+    task_type_ = model.task_type;
+    task_param_ = model.task_param;
     pred_transform_ = model.param.pred_transform;
     sigmoid_alpha_ = model.param.sigmoid_alpha;
     global_bias_ = model.param.global_bias;
-    pred_tranform_func_ = PredTransformFunction("native", model);
     files_.clear();
 
-    ASTBuilder builder;
+    ASTBuilder<ThresholdType, LeafOutputType> builder;
     builder.BuildAST(model);
-    if (builder.FoldCode(param.code_folding_req)
-        || param.quantize > 0) {
+    if (builder.FoldCode(param_.code_folding_req) || param_.quantize > 0) {
       // is_categorical[i] : is i-th feature categorical?
       array_is_categorical_
         = RenderIsCategoricalArray(builder.GenerateIsCategoricalArray());
     }
-    if (param.annotate_in != "NULL") {
+    if (param_.annotate_in != "NULL") {
       BranchAnnotator annotator;
-      std::unique_ptr<dmlc::Stream> fi(
-        dmlc::Stream::Create(param.annotate_in.c_str(), "r"));
-      annotator.Load(fi.get());
+      std::ifstream fi(param_.annotate_in.c_str());
+      annotator.Load(fi);
       const auto annotation = annotator.Get();
       builder.LoadDataCounts(annotation);
-      LOG(INFO) << "Loading node frequencies from `"
-                << param.annotate_in << "'";
+      TREELITE_LOG(INFO) << "Loading node frequencies from `"
+                         << param_.annotate_in << "'";
     }
-    builder.Split(param.parallel_comp);
-    if (param.quantize > 0) {
+    builder.Split(param_.parallel_comp);
+    if (param_.quantize > 0) {
       builder.QuantizeThresholds();
     }
 
@@ -92,39 +90,61 @@ class ASTNativeCompiler : public Compiler {
       }
     }
 
-    WalkAST(builder.GetRootNode(), "main.c", 0);
+    WalkAST<ThresholdType, LeafOutputType>(builder.GetRootNode(), "main.c", 0);
     if (files_.count("arrays.c") > 0) {
       PrependToBuffer("arrays.c", "#include \"header.h\"\n", 0);
     }
 
     {
       /* write recipe.json */
-      std::vector<std::unordered_map<std::string, std::string>> source_list;
+      rapidjson::StringBuffer os;
+      rapidjson::Writer<rapidjson::StringBuffer> writer(os);
+
+      writer.StartObject();
+      writer.Key("target");
+      writer.String(param_.native_lib_name.data(), param_.native_lib_name.size());
+      writer.Key("sources");
+      writer.StartArray();
       for (const auto& kv : files_) {
         if (kv.first.compare(kv.first.length() - 2, 2, ".c") == 0) {
           const size_t line_count
             = std::count(kv.second.content.begin(), kv.second.content.end(), '\n');
-          source_list.push_back({ {"name",
-                                   kv.first.substr(0, kv.first.length() - 2)},
-                                  {"length", std::to_string(line_count)} });
+          writer.StartObject();
+          writer.Key("name");
+          std::string name = kv.first.substr(0, kv.first.length() - 2);
+          writer.String(name.data(), name.size());
+          writer.Key("length");
+          writer.Uint64(line_count);
+          writer.EndObject();
         }
       }
-      std::ostringstream oss;
-      std::unique_ptr<dmlc::JSONWriter> writer(new dmlc::JSONWriter(&oss));
-      writer->BeginObject();
-      writer->WriteObjectKeyValue("target", param.native_lib_name);
-      writer->WriteObjectKeyValue("sources", source_list);
-      writer->EndObject();
-      files_["recipe.json"] = CompiledModel::FileEntry(oss.str());
+      writer.EndArray();
+      writer.EndObject();
+
+      files_["recipe.json"] = CompiledModel::FileEntry(os.GetString());
     }
     cm.files = std::move(files_);
     return cm;
   }
 
+  CompiledModel Compile(const Model& model) {
+    TREELITE_CHECK(model.GetLeafOutputType() != TypeInfo::kUInt32)
+      << "Integer leaf outputs not yet supported";
+    this->pred_tranform_func_ = PredTransformFunction("native", model);
+    return model.Dispatch([this](const auto& model_handle) {
+      return this->CompileImpl(model_handle);
+    });
+  }
+
+  CompilerParam QueryParam() const {
+    return param_;
+  }
+
  private:
-  CompilerParam param;
+  CompilerParam param_;
   int num_feature_;
-  int num_output_group_;
+  TaskType task_type_;
+  TaskParam task_param_;
   std::string pred_transform_;
   float sigmoid_alpha_;
   float global_bias_;
@@ -132,32 +152,33 @@ class ASTNativeCompiler : public Compiler {
   std::string array_is_categorical_;
   std::unordered_map<std::string, CompiledModel::FileEntry> files_;
 
+  template <typename ThresholdType, typename LeafOutputType>
   void WalkAST(const ASTNode* node,
                const std::string& dest,
                size_t indent) {
     const MainNode* t1;
     const AccumulatorContextNode* t2;
     const ConditionNode* t3;
-    const OutputNode* t4;
+    const OutputNode<LeafOutputType>* t4;
     const TranslationUnitNode* t5;
-    const QuantizerNode* t6;
+    const QuantizerNode<ThresholdType>* t6;
     const CodeFolderNode* t7;
     if ( (t1 = dynamic_cast<const MainNode*>(node)) ) {
-      HandleMainNode(t1, dest, indent);
+      HandleMainNode<ThresholdType, LeafOutputType>(t1, dest, indent);
     } else if ( (t2 = dynamic_cast<const AccumulatorContextNode*>(node)) ) {
-      HandleACNode(t2, dest, indent);
+      HandleACNode<ThresholdType, LeafOutputType>(t2, dest, indent);
     } else if ( (t3 = dynamic_cast<const ConditionNode*>(node)) ) {
-      HandleCondNode(t3, dest, indent);
-    } else if ( (t4 = dynamic_cast<const OutputNode*>(node)) ) {
-      HandleOutputNode(t4, dest, indent);
+      HandleCondNode<ThresholdType, LeafOutputType>(t3, dest, indent);
+    } else if ( (t4 = dynamic_cast<const OutputNode<LeafOutputType>*>(node)) ) {
+      HandleOutputNode<ThresholdType, LeafOutputType>(t4, dest, indent);
     } else if ( (t5 = dynamic_cast<const TranslationUnitNode*>(node)) ) {
-      HandleTUNode(t5, dest, indent);
-    } else if ( (t6 = dynamic_cast<const QuantizerNode*>(node)) ) {
-      HandleQNode(t6, dest, indent);
+      HandleTUNode<ThresholdType, LeafOutputType>(t5, dest, indent);
+    } else if ( (t6 = dynamic_cast<const QuantizerNode<ThresholdType>*>(node)) ) {
+      HandleQNode<ThresholdType, LeafOutputType>(t6, dest, indent);
     } else if ( (t7 = dynamic_cast<const CodeFolderNode*>(node)) ) {
-      HandleCodeFolderNode(t7, dest, indent);
+      HandleCodeFolderNode<ThresholdType, LeafOutputType>(t7, dest, indent);
     } else {
-      LOG(FATAL) << "Unrecognized AST node type";
+      TREELITE_LOG(FATAL) << "Unrecognized AST node type";
     }
   }
 
@@ -176,24 +197,20 @@ class ASTNativeCompiler : public Compiler {
       = common_util::IndentMultiLineString(content, indent) + files_[dest].content;
   }
 
+  template <typename ThresholdType, typename LeafOutputType>
   void HandleMainNode(const MainNode* node,
                       const std::string& dest,
                       size_t indent) {
-    const char* get_num_output_group_function_signature
-      = "size_t get_num_output_group(void)";
-    const char* get_num_feature_function_signature
-      = "size_t get_num_feature(void)";
-    const char* get_pred_transform_function_signature
-      = "const char* get_pred_transform(void)";
-    const char* get_sigmoid_alpha_function_signature
-      = "float get_sigmoid_alpha(void)";
-    const char* get_global_bias_function_signature
-      = "float get_global_bias(void)";
-    const char* predict_function_signature
-      = (num_output_group_ > 1) ?
-          "size_t predict_multiclass(union Entry* data, int pred_margin, "
-                                    "float* result)"
-        : "float predict(union Entry* data, int pred_margin)";
+    const std::string threshold_type
+      = native::TypeInfoToCTypeString(TypeToInfo<ThresholdType>());
+    const std::string leaf_output_type
+      = native::TypeInfoToCTypeString(TypeToInfo<LeafOutputType>());
+    const std::string predict_function_signature
+      = (task_param_.num_class > 1) ?
+          fmt::format("size_t predict_multiclass(union Entry* data, int pred_margin, {}* result)",
+                      leaf_output_type)
+        : fmt::format("{} predict(union Entry* data, int pred_margin)",
+                      leaf_output_type);
 
     if (!array_is_categorical_.empty()) {
       array_is_categorical_
@@ -201,94 +218,109 @@ class ASTNativeCompiler : public Compiler {
                       array_is_categorical_);
     }
 
+    const std::string query_functions_definition
+      = fmt::format(native::query_functions_definition_template,
+          "num_class"_a = task_param_.num_class,
+          "num_feature"_a = num_feature_,
+          "pred_transform"_a = pred_transform_,
+          "sigmoid_alpha"_a = sigmoid_alpha_,
+          "global_bias"_a = global_bias_,
+          "threshold_type_str"_a = TypeInfoToString(TypeToInfo<ThresholdType>()),
+          "leaf_output_type_str"_a = TypeInfoToString(TypeToInfo<LeafOutputType>()));
+
     AppendToBuffer(dest,
       fmt::format(native::main_start_template,
         "array_is_categorical"_a = array_is_categorical_,
-        "get_num_output_group_function_signature"_a
-          = get_num_output_group_function_signature,
-        "get_num_feature_function_signature"_a
-          = get_num_feature_function_signature,
-        "get_pred_transform_function_signature"_a
-          = get_pred_transform_function_signature,
-        "get_sigmoid_alpha_function_signature"_a
-          = get_sigmoid_alpha_function_signature,
-        "get_global_bias_function_signature"_a
-          = get_global_bias_function_signature,
+        "query_functions_definition"_a = query_functions_definition,
         "pred_transform_function"_a = pred_tranform_func_,
-        "predict_function_signature"_a = predict_function_signature,
-        "num_output_group"_a = num_output_group_,
-        "num_feature"_a = num_feature_,
-        "pred_transform"_a = pred_transform_,
-        "sigmoid_alpha"_a = sigmoid_alpha_,
-        "global_bias"_a = global_bias_),
+        "predict_function_signature"_a = predict_function_signature),
       indent);
+    const std::string query_functions_prototype
+      = fmt::format(native::query_functions_prototype_template,
+          "dllexport"_a = DLLEXPORT_KEYWORD);
     AppendToBuffer("header.h",
       fmt::format(native::header_template,
         "dllexport"_a = DLLEXPORT_KEYWORD,
-        "get_num_output_group_function_signature"_a
-          = get_num_output_group_function_signature,
-        "get_num_feature_function_signature"_a
-          = get_num_feature_function_signature,
-        "get_pred_transform_function_signature"_a
-          = get_pred_transform_function_signature,
-        "get_sigmoid_alpha_function_signature"_a
-          = get_sigmoid_alpha_function_signature,
-        "get_global_bias_function_signature"_a
-          = get_global_bias_function_signature,
         "predict_function_signature"_a = predict_function_signature,
-        "threshold_type"_a = (param.quantize > 0 ? "int" : "float")),
+        "query_functions_prototype"_a = query_functions_prototype,
+        "threshold_type"_a = threshold_type,
+        "threshold_type_Node"_a = (param_.quantize > 0 ? std::string("int") : threshold_type)),
       indent);
 
-    CHECK_EQ(node->children.size(), 1);
-    WalkAST(node->children[0], dest, indent + 2);
+    TREELITE_CHECK_EQ(node->children.size(), 1);
+    WalkAST<ThresholdType, LeafOutputType>(node->children[0], dest, indent + 2);
 
-    const std::string optional_average_field
-      = (node->average_result) ? fmt::format(" / {}", node->num_tree)
-                               : std::string("");
-    if (num_output_group_ > 1) {
+    std::string optional_average_field;
+    if (node->average_result) {
+      if (task_type_ == TaskType::kMultiClfGrovePerClass) {
+        TREELITE_CHECK(task_param_.grove_per_class);
+        TREELITE_CHECK_EQ(task_param_.leaf_vector_size, 1);
+        TREELITE_CHECK_GT(task_param_.num_class, 1);
+        TREELITE_CHECK_EQ(node->num_tree % task_param_.num_class, 0)
+          << "Expected the number of trees to be divisible by the number of classes";
+        int num_boosting_round = node->num_tree / static_cast<int>(task_param_.num_class);
+        optional_average_field = fmt::format(" / {}", num_boosting_round);
+      } else {
+        TREELITE_CHECK(task_type_ == TaskType::kBinaryClfRegr
+                       || task_type_ == TaskType::kMultiClfProbDistLeaf);
+        TREELITE_CHECK_EQ(task_param_.num_class, task_param_.leaf_vector_size);
+        TREELITE_CHECK(!task_param_.grove_per_class);
+        optional_average_field = fmt::format(" / {}", node->num_tree);
+      }
+    }
+    if (task_param_.num_class > 1) {
       AppendToBuffer(dest,
         fmt::format(native::main_end_multiclass_template,
-          "num_output_group"_a = num_output_group_,
+          "num_class"_a = task_param_.num_class,
           "optional_average_field"_a = optional_average_field,
-          "global_bias"_a = common_util::ToStringHighPrecision(node->global_bias)),
+          "global_bias"_a = common_util::ToStringHighPrecision(node->global_bias),
+          "leaf_output_type"_a = leaf_output_type),
         indent);
     } else {
       AppendToBuffer(dest,
         fmt::format(native::main_end_template,
           "optional_average_field"_a = optional_average_field,
-          "global_bias"_a = common_util::ToStringHighPrecision(node->global_bias)),
+          "global_bias"_a = common_util::ToStringHighPrecision(node->global_bias),
+          "leaf_output_type"_a = leaf_output_type),
         indent);
     }
   }
 
+  template <typename ThresholdType, typename LeafOutputType>
   void HandleACNode(const AccumulatorContextNode* node,
                     const std::string& dest,
                     size_t indent) {
-    if (num_output_group_ > 1) {
+    const std::string leaf_output_type
+      = native::TypeInfoToCTypeString(TypeToInfo<LeafOutputType>());
+    if (task_param_.num_class > 1) {
       AppendToBuffer(dest,
-        fmt::format("float sum[{num_output_group}] = {{0.0f}};\n"
+        fmt::format("{leaf_output_type} sum[{num_class}] = {{0}};\n"
                     "unsigned int tmp;\n"
                     "int nid, cond, fid;  /* used for folded subtrees */\n",
-          "num_output_group"_a = num_output_group_), indent);
+          "num_class"_a = task_param_.num_class,
+          "leaf_output_type"_a = leaf_output_type), indent);
     } else {
       AppendToBuffer(dest,
-        "float sum = 0.0f;\n"
-        "unsigned int tmp;\n"
-        "int nid, cond, fid;  /* used for folded subtrees */\n", indent);
+        fmt::format("{leaf_output_type} sum = ({leaf_output_type})0;\n"
+                    "unsigned int tmp;\n"
+                    "int nid, cond, fid;  /* used for folded subtrees */\n",
+          "leaf_output_type"_a = leaf_output_type),
+        indent);
     }
     for (ASTNode* child : node->children) {
-      WalkAST(child, dest, indent);
+      WalkAST<ThresholdType, LeafOutputType>(child, dest, indent);
     }
   }
 
+  template <typename ThresholdType, typename LeafOutputType>
   void HandleCondNode(const ConditionNode* node,
                       const std::string& dest,
                       size_t indent) {
-    const NumericalConditionNode* t;
-    std::string condition, condition_with_na_check;
-    if ( (t = dynamic_cast<const NumericalConditionNode*>(node)) ) {
+    const NumericalConditionNode<ThresholdType>* t;
+    std::string condition_with_na_check;
+    if ( (t = dynamic_cast<const NumericalConditionNode<ThresholdType>*>(node)) ) {
       /* numerical split */
-      condition = ExtractNumericalCondition(t);
+      std::string condition = ExtractNumericalCondition(t);
       const char* condition_with_na_check_template
         = (node->default_left) ?
             "!(data[{split_index}].missing != -1) || ({condition})"
@@ -298,14 +330,13 @@ class ASTNativeCompiler : public Compiler {
             "split_index"_a = node->split_index,
             "condition"_a = condition);
     } else {   /* categorical split */
-      const CategoricalConditionNode* t2
-        = dynamic_cast<const CategoricalConditionNode*>(node);
-      CHECK(t2);
+      const CategoricalConditionNode* t2 = dynamic_cast<const CategoricalConditionNode*>(node);
+      TREELITE_CHECK(t2);
       condition_with_na_check = ExtractCategoricalCondition(t2);
     }
     if (node->children[0]->data_count && node->children[1]->data_count) {
-      const int left_freq = node->children[0]->data_count.value();
-      const int right_freq = node->children[1]->data_count.value();
+      const uint64_t left_freq = *node->children[0]->data_count;
+      const uint64_t right_freq = *node->children[1]->data_count;
       condition_with_na_check
         = fmt::format(" {keyword}( {condition} ) ",
             "keyword"_a = ((left_freq > right_freq) ? "LIKELY" : "UNLIKELY"),
@@ -313,41 +344,48 @@ class ASTNativeCompiler : public Compiler {
     }
     AppendToBuffer(dest,
       fmt::format("if ({}) {{\n", condition_with_na_check), indent);
-    CHECK_EQ(node->children.size(), 2);
-    WalkAST(node->children[0], dest, indent + 2);
+    TREELITE_CHECK_EQ(node->children.size(), 2);
+    WalkAST<ThresholdType, LeafOutputType>(node->children[0], dest, indent + 2);
     AppendToBuffer(dest, "} else {\n", indent);
-    WalkAST(node->children[1], dest, indent + 2);
+    WalkAST<ThresholdType, LeafOutputType>(node->children[1], dest, indent + 2);
     AppendToBuffer(dest, "}\n", indent);
   }
 
-  void HandleOutputNode(const OutputNode* node,
+  template <typename ThresholdType, typename LeafOutputType>
+  void HandleOutputNode(const OutputNode<LeafOutputType>* node,
                         const std::string& dest,
                         size_t indent) {
     AppendToBuffer(dest, RenderOutputStatement(node), indent);
-    CHECK_EQ(node->children.size(), 0);
+    TREELITE_CHECK_EQ(node->children.size(), 0);
   }
 
+  template <typename ThresholdType, typename LeafOutputType>
   void HandleTUNode(const TranslationUnitNode* node,
                     const std::string& dest,
-                    int indent) {
+                    size_t indent) {
     const int unit_id = node->unit_id;
     const std::string new_file = fmt::format("tu{}.c", unit_id);
+    const std::string leaf_output_type
+      = native::TypeInfoToCTypeString(TypeToInfo<LeafOutputType>());
 
     std::string unit_function_name, unit_function_signature,
                 unit_function_call_signature;
-    if (num_output_group_ > 1) {
+    if (task_param_.num_class > 1) {
       unit_function_name
         = fmt::format("predict_margin_multiclass_unit{}", unit_id);
       unit_function_signature
-        = fmt::format("void {}(union Entry* data, float* result)",
-            unit_function_name);
+        = fmt::format("void {function_name}(union Entry* data, {leaf_output_type}* result)",
+            "function_name"_a = unit_function_name,
+            "leaf_output_type"_a = leaf_output_type);
       unit_function_call_signature
         = fmt::format("{}(data, sum);\n", unit_function_name);
     } else {
       unit_function_name
         = fmt::format("predict_margin_unit{}", unit_id);
       unit_function_signature
-        = fmt::format("float {}(union Entry* data)", unit_function_name);
+        = fmt::format("{leaf_output_type} {function_name}(union Entry* data)",
+            "function_name"_a = unit_function_name,
+            "leaf_output_type"_a = leaf_output_type);
       unit_function_call_signature
         = fmt::format("sum += {}(data);\n", unit_function_name);
     }
@@ -355,24 +393,27 @@ class ASTNativeCompiler : public Compiler {
     AppendToBuffer(new_file,
                    fmt::format("#include \"header.h\"\n"
                                "{} {{\n", unit_function_signature), 0);
-    CHECK_EQ(node->children.size(), 1);
-    WalkAST(node->children[0], new_file, 2);
-    if (num_output_group_ > 1) {
+    TREELITE_CHECK_EQ(node->children.size(), 1);
+    WalkAST<ThresholdType, LeafOutputType>(node->children[0], new_file, 2);
+    if (task_param_.num_class > 1) {
       AppendToBuffer(new_file,
-        fmt::format("  for (int i = 0; i < {num_output_group}; ++i) {{\n"
+        fmt::format("  for (int i = 0; i < {num_class}; ++i) {{\n"
                     "    result[i] += sum[i];\n"
                     "  }}\n"
                     "}}\n",
-          "num_output_group"_a = num_output_group_), 0);
+          "num_class"_a = task_param_.num_class), 0);
     } else {
       AppendToBuffer(new_file, "  return sum;\n}\n", 0);
     }
     AppendToBuffer("header.h", fmt::format("{};\n", unit_function_signature), 0);
   }
 
-  void HandleQNode(const QuantizerNode* node,
+  template <typename ThresholdType, typename LeafOutputType>
+  void HandleQNode(const QuantizerNode<ThresholdType>* node,
                    const std::string& dest,
                    size_t indent) {
+    const std::string threshold_type
+      = native::TypeInfoToCTypeString(TypeToInfo<ThresholdType>());
     /* render arrays needed to convert feature values into bin indices */
     std::string array_threshold, array_th_begin, array_th_len;
     // threshold[] : list of all thresholds that occur at least once in the
@@ -386,7 +427,7 @@ class ASTNativeCompiler : public Compiler {
       for (const auto& e : node->cut_pts) {
         // cut_pts had been generated in ASTBuilder::QuantizeThresholds
         // cut_pts[i][k] stores the k-th threshold of feature i.
-        for (tl_float v : e) {
+        for (auto v : e) {
           formatter << v;
         }
       }
@@ -412,16 +453,21 @@ class ASTNativeCompiler : public Compiler {
     if (!array_threshold.empty() && !array_th_begin.empty() && !array_th_len.empty()) {
       PrependToBuffer(dest,
         fmt::format(native::qnode_template,
-          "total_num_threshold"_a = total_num_threshold), 0);
+          "total_num_threshold"_a = total_num_threshold,
+          "threshold_type"_a = threshold_type),
+        0);
       AppendToBuffer(dest,
         fmt::format(native::quantize_loop_template,
           "num_feature"_a = num_feature_), indent);
     }
     if (!array_threshold.empty()) {
       PrependToBuffer(dest,
-        fmt::format("static const float threshold[] = {{\n"
+        fmt::format("static const {threshold_type} threshold[] = {{\n"
                     "{array_threshold}\n"
-                    "}};\n", "array_threshold"_a = array_threshold), 0);
+                    "}};\n",
+          "array_threshold"_a = array_threshold,
+          "threshold_type"_a = threshold_type),
+        0);
     }
     if (!array_th_begin.empty()) {
       PrependToBuffer(dest,
@@ -435,14 +481,15 @@ class ASTNativeCompiler : public Compiler {
                     "{array_th_len}\n"
                     "}};\n", "array_th_len"_a = array_th_len), 0);
     }
-    CHECK_EQ(node->children.size(), 1);
-    WalkAST(node->children[0], dest, indent);
+    TREELITE_CHECK_EQ(node->children.size(), 1);
+    WalkAST<ThresholdType, LeafOutputType>(node->children[0], dest, indent);
   }
 
+  template <typename ThresholdType, typename LeafOutputType>
   void HandleCodeFolderNode(const CodeFolderNode* node,
                             const std::string& dest,
                             size_t indent) {
-    CHECK_EQ(node->children.size(), 1);
+    TREELITE_CHECK_EQ(node->children.size(), 1);
     const int node_id = node->children[0]->node_id;
     const int tree_id = node->children[0]->tree_id;
 
@@ -464,11 +511,11 @@ class ASTNativeCompiler : public Compiler {
 
     std::string output_switch_statement;
     Operator common_comp_op;
-    common_util::RenderCodeFolderArrays(node, param.quantize, false,
-      "{{ {default_left}, {split_index}, {threshold}, {left_child}, {right_child} }}",
-      [this](const OutputNode* node) { return RenderOutputStatement(node); },
-      &array_nodes, &array_cat_bitmap, &array_cat_begin,
-      &output_switch_statement, &common_comp_op);
+    common_util::RenderCodeFolderArrays<ThresholdType, LeafOutputType>(node, param_.quantize,
+      false, "{{ {default_left}, {split_index}, {threshold}, {left_child}, {right_child} }}",
+      [this](const OutputNode<LeafOutputType>* node) { return RenderOutputStatement(node); },
+      &array_nodes, &array_cat_bitmap, &array_cat_begin, &output_switch_statement,
+      &common_comp_op);
     if (!array_nodes.empty()) {
       AppendToBuffer("header.h",
                      fmt::format("extern const struct Node {node_array_name}[];\n",
@@ -518,7 +565,7 @@ class ASTNativeCompiler : public Compiler {
                        "node_array_name"_a = node_array_name,
                        "cat_bitmap_name"_a = cat_bitmap_name,
                        "cat_begin_name"_a = cat_begin_name,
-                       "data_field"_a = (param.quantize > 0 ? "qvalue" : "fvalue"),
+                       "data_field"_a = (param_.quantize > 0 ? "qvalue" : "fvalue"),
                        "comp_op"_a = OpName(common_comp_op),
                        "output_switch_statement"_a
                          = output_switch_statement), indent);
@@ -526,30 +573,40 @@ class ASTNativeCompiler : public Compiler {
       AppendToBuffer(dest,
                      fmt::format(native::eval_loop_template_without_categorical_feature,
                        "node_array_name"_a = node_array_name,
-                       "data_field"_a = (param.quantize > 0 ? "qvalue" : "fvalue"),
+                       "data_field"_a = (param_.quantize > 0 ? "qvalue" : "fvalue"),
                        "comp_op"_a = OpName(common_comp_op),
                        "output_switch_statement"_a
                          = output_switch_statement), indent);
     }
   }
 
+  template <typename ThresholdType>
   inline std::string
-  ExtractNumericalCondition(const NumericalConditionNode* node) {
+  ExtractNumericalCondition(const NumericalConditionNode<ThresholdType>* node) {
+    const std::string threshold_type
+        = native::TypeInfoToCTypeString(TypeToInfo<ThresholdType>());
     std::string result;
     if (node->quantized) {  // quantized threshold
-      result = fmt::format("data[{split_index}].qvalue {opname} {threshold}",
-                 "split_index"_a = node->split_index,
+      std::string lhs = fmt::format("data[{split_index}].qvalue",
+                                    "split_index"_a = node->split_index);
+      result = fmt::format("{lhs} {opname} {threshold}",
+                 "lhs"_a = lhs,
                  "opname"_a = OpName(node->op),
                  "threshold"_a = node->threshold.int_val);
     } else if (std::isinf(node->threshold.float_val)) {  // infinite threshold
       // According to IEEE 754, the result of comparison [lhs] < infinity
       // must be identical for all finite [lhs]. Same goes for operator >.
-      result = (CompareWithOp(0.0, node->op, node->threshold.float_val) ? "1" : "0");
+      result = (CompareWithOp(static_cast<ThresholdType>(0), node->op, node->threshold.float_val)
+          ? "1" : "0");
     } else {  // finite threshold
-      result = fmt::format("data[{split_index}].fvalue {opname} (float){threshold}",
-                 "split_index"_a = node->split_index,
-                 "opname"_a = OpName(node->op),
-                 "threshold"_a = common_util::ToStringHighPrecision(node->threshold.float_val));
+      std::string lhs = fmt::format("data[{split_index}].fvalue",
+                                    "split_index"_a = node->split_index);
+      result
+        = fmt::format("{lhs} {opname} ({threshold_type}){threshold}",
+            "lhs"_a = lhs,
+            "opname"_a = OpName(node->op),
+            "threshold_type"_a = threshold_type,
+            "threshold"_a = common_util::ToStringHighPrecision(node->threshold.float_val));
     }
     return result;
   }
@@ -558,8 +615,8 @@ class ASTNativeCompiler : public Compiler {
   ExtractCategoricalCondition(const CategoricalConditionNode* node) {
     std::string result;
     std::vector<uint64_t> bitmap
-      = common_util::GetCategoricalBitmap(node->left_categories);
-    CHECK_GE(bitmap.size(), 1);
+      = common_util::GetCategoricalBitmap(node->matching_categories);
+    TREELITE_CHECK_GE(bitmap.size(), 1);
     bool all_zeros = true;
     for (uint64_t e : bitmap) {
       all_zeros &= (e == 0);
@@ -568,21 +625,19 @@ class ASTNativeCompiler : public Compiler {
       result = "0";
     } else {
       std::ostringstream oss;
-      if (node->convert_missing_to_zero) {
-        // All missing values are converted into zeros
+      const std::string right_categories_flag = (node->categories_list_right_child ? "!" : "");
+      if (node->default_left) {
         oss << fmt::format(
-          "((tmp = (data[{0}].missing == -1 ? 0U "
-          ": (unsigned int)(data[{0}].fvalue) )), ", node->split_index);
+          "data[{split_index}].missing == -1 || {right_categories_flag}("
+          "(tmp = (unsigned int)(data[{split_index}].fvalue) ), ",
+          "split_index"_a = node->split_index,
+          "right_categories_flag"_a = right_categories_flag);
       } else {
-        if (node->default_left) {
-          oss << fmt::format(
-            "data[{0}].missing == -1 || ("
-            "(tmp = (unsigned int)(data[{0}].fvalue) ), ", node->split_index);
-        } else {
-          oss << fmt::format(
-            "data[{0}].missing != -1 && ("
-            "(tmp = (unsigned int)(data[{0}].fvalue) ), ", node->split_index);
-        }
+        oss << fmt::format(
+          "data[{split_index}].missing != -1 && {right_categories_flag}("
+          "(tmp = (unsigned int)(data[{split_index}].fvalue) ), ",
+          "split_index"_a = node->split_index,
+          "right_categories_flag"_a = right_categories_flag);
       }
       oss << "(tmp >= 0 && tmp < 64 && (( (uint64_t)"
           << bitmap[0] << "U >> tmp) & 1) )";
@@ -607,39 +662,63 @@ class ASTNativeCompiler : public Compiler {
     return formatter.str();
   }
 
-  inline std::string RenderOutputStatement(const OutputNode* node) {
+  template <typename LeafOutputType>
+  inline std::string RenderOutputStatement(const OutputNode<LeafOutputType>* node) {
+    const std::string leaf_output_type
+      = native::TypeInfoToCTypeString(TypeToInfo<LeafOutputType>());
     std::string output_statement;
-    if (num_output_group_ > 1) {
+    if (task_param_.num_class > 1) {
       if (node->is_vector) {
         // multi-class classification with random forest
-        CHECK_EQ(node->vector.size(), static_cast<size_t>(num_output_group_))
-          << "Ill-formed model: leaf vector must be of length [num_output_group]";
-        for (int group_id = 0; group_id < num_output_group_; ++group_id) {
+        TREELITE_CHECK_EQ(node->vector.size(), static_cast<size_t>(task_param_.num_class))
+          << "Ill-formed model: leaf vector must be of length [num_class]";
+        for (size_t group_id = 0; group_id < task_param_.num_class; ++group_id) {
           output_statement
-            += fmt::format("sum[{group_id}] += (float){output};\n",
+            += fmt::format("sum[{group_id}] += ({leaf_output_type}){output};\n",
                  "group_id"_a = group_id,
-                 "output"_a = common_util::ToStringHighPrecision(node->vector[group_id]));
+                 "output"_a = common_util::ToStringHighPrecision(node->vector[group_id]),
+                 "leaf_output_type"_a = leaf_output_type);
         }
       } else {
         // multi-class classification with gradient boosted trees
         output_statement
-          = fmt::format("sum[{group_id}] += (float){output};\n",
-              "group_id"_a = node->tree_id % num_output_group_,
-              "output"_a = common_util::ToStringHighPrecision(node->scalar));
+          = fmt::format("sum[{group_id}] += ({leaf_output_type}){output};\n",
+              "group_id"_a = node->tree_id % task_param_.num_class,
+              "output"_a = common_util::ToStringHighPrecision(node->scalar),
+              "leaf_output_type"_a = leaf_output_type);
       }
     } else {
       output_statement
-        = fmt::format("sum += (float){output};\n",
-            "output"_a = common_util::ToStringHighPrecision(node->scalar));
+        = fmt::format("sum += ({leaf_output_type}){output};\n",
+            "output"_a = common_util::ToStringHighPrecision(node->scalar),
+            "leaf_output_type"_a = leaf_output_type);
     }
     return output_statement;
   }
 };
 
-TREELITE_REGISTER_COMPILER(ASTNativeCompiler, "ast_native")
-.describe("AST-based compiler that produces C code")
-.set_body([](const CompilerParam& param) -> Compiler* {
-    return new ASTNativeCompiler(param);
-  });
+ASTNativeCompiler::ASTNativeCompiler(const CompilerParam& param)
+    : pimpl_(std::make_unique<ASTNativeCompilerImpl>(param)) {
+  if (param.verbose > 0) {
+    TREELITE_LOG(INFO) << "Using ASTNativeCompiler";
+  }
+  if (param.dump_array_as_elf > 0) {
+    TREELITE_LOG(INFO) << "Warning: 'dump_array_as_elf' parameter is not applicable "
+                 "for ASTNativeCompiler";
+  }
+}
+
+ASTNativeCompiler::~ASTNativeCompiler() = default;
+
+CompiledModel
+ASTNativeCompiler::Compile(const Model &model) {
+  return pimpl_->Compile(model);
+}
+
+CompilerParam
+ASTNativeCompiler::QueryParam() const {
+  return pimpl_->QueryParam();
+}
+
 }  // namespace compiler
 }  // namespace treelite
