@@ -26,12 +26,14 @@ FUNC_NAME(const std::string& backend, const Model& model) { \
   if (backend == "native") { \
     return treelite::compiler::native::pred_transform::FUNC_NAME(model); \
   } else { \
-    LOG(FATAL) << "Unrecognized backend: " << backend; \
+    TREELITE_LOG(FATAL) << "Unrecognized backend: " << backend; \
     return std::string(); \
   } \
 }
 
 TREELITE_PRED_TRANSFORM_REGISTRY_DEFAULT_TEMPLATE(identity)
+TREELITE_PRED_TRANSFORM_REGISTRY_DEFAULT_TEMPLATE(signed_square)
+TREELITE_PRED_TRANSFORM_REGISTRY_DEFAULT_TEMPLATE(hinge)
 TREELITE_PRED_TRANSFORM_REGISTRY_DEFAULT_TEMPLATE(sigmoid)
 TREELITE_PRED_TRANSFORM_REGISTRY_DEFAULT_TEMPLATE(exponential)
 TREELITE_PRED_TRANSFORM_REGISTRY_DEFAULT_TEMPLATE(logarithm_one_plus_exp)
@@ -43,23 +45,31 @@ TREELITE_PRED_TRANSFORM_REGISTRY_DEFAULT_TEMPLATE(multiclass_ova)
 const std::unordered_map<std::string, PredTransformFuncGenerator>
 pred_transform_db = {
   PRED_TRANSFORM_FUNC(identity),
+  PRED_TRANSFORM_FUNC(signed_square),
+  PRED_TRANSFORM_FUNC(hinge),
   PRED_TRANSFORM_FUNC(sigmoid),
   PRED_TRANSFORM_FUNC(exponential),
   PRED_TRANSFORM_FUNC(logarithm_one_plus_exp)
 };
 /*! [pred_transform_db]
   - identity
-    do not transform. The output will be a vector of length
+    Do not transform. The output will be a vector of length
     [number of data points] that contains the margin score for every data point.
+  - signed_square
+    Apply the function f(x) = sign(x) * (x**2) element-wise to the margin vector. The
+    output will be a vector of length [number of data points].
+  - hinge
+    Apply the function f(x) = (1 if x > 0 else 0) element-wise to the margin vector. The
+    output will be a vector of length [number of data points], filled with 0's and 1's.
   - sigmoid
-    apply the sigmoid function element-wise to the margin vector. The output
+    Apply the sigmoid function element-wise to the margin vector. The output
     will be a vector of length [number of data points] that contains the
     probability of each data point belonging to the positive class.
   - exponential
-    apply the exponential function (exp) element-wise to the margin vector. The
+    Apply the exponential function (exp) element-wise to the margin vector. The
     output will be a vector of length [number of data points].
   - logarithm_one_plus_exp
-    apply the function f(x) = log(1 + exp(x)) element-wise to the margin vector.
+    Apply the function f(x) = log(1 + exp(x)) element-wise to the margin vector.
     The output will be a vector of length [number of data points].
     [pred_transform_db] */
 
@@ -95,30 +105,31 @@ pred_transform_multiclass_db = {
 std::string
 treelite::compiler::PredTransformFunction(const std::string& backend,
                                           const Model& model) {
-  if (model.num_output_group > 1) {  // multi-class classification
-    auto it = pred_transform_multiclass_db.find(model.param.pred_transform);
+  ModelParam param = model.param;
+  if (model.task_param.num_class > 1) {  // multi-class classification
+    auto it = pred_transform_multiclass_db.find(param.pred_transform);
     if (it == pred_transform_multiclass_db.end()) {
       std::ostringstream oss;
       for (const auto& e : pred_transform_multiclass_db) {
         oss << "'" << e.first << "', ";
       }
-      LOG(FATAL) << "Invalid argument given for `pred_transform` parameter. "
-                 << "For multi-class classification, you should set "
-                 << "`pred_transform` to one of the following: "
-                 << "{ " << oss.str() << " }";
+      TREELITE_LOG(FATAL) << "Invalid argument given for `pred_transform` parameter. "
+                          << "For multi-class classification, you should set "
+                          << "`pred_transform` to one of the following: "
+                          << "{ " << oss.str() << " }";
     }
     return (it->second)(backend, model);
   } else {
-    auto it = pred_transform_db.find(model.param.pred_transform);
+    auto it = pred_transform_db.find(param.pred_transform);
     if (it == pred_transform_db.end()) {
       std::ostringstream oss;
       for (const auto& e : pred_transform_db) {
         oss << "'" << e.first << "', ";
       }
-      LOG(FATAL) << "Invalid argument given for `pred_transform` parameter. "
-                 << "For any task that is NOT multi-class classification, you "
-                 << "should set `pred_transform` to one of the following: "
-                 << "{ " << oss.str() << " }";
+      TREELITE_LOG(FATAL) << "Invalid argument given for `pred_transform` parameter. "
+                          << "For any task that is NOT multi-class classification, you "
+                          << "should set `pred_transform` to one of the following: "
+                          << "{ " << oss.str() << " }";
     }
     return (it->second)(backend, model);
   }
